@@ -116,14 +116,39 @@ class TestAffine2D:
         assert Affine2D.identity().scale(1, 0).is_degenerate()
         assert Affine2D.identity().scale(0, 0).is_degenerate()
 
-    def test_inverse(self):
-        t = Affine2D.identity().translate(2, 3).scale(4, 5)
+    @pytest.mark.parametrize(
+        "transform, inverse",
+        [
+            (
+                Affine2D.identity().translate(2, 3).scale(4, 5),
+                Affine2D(a=0.25, b=0.0, c=0.0, d=0.2, e=-0.5, f=-0.6),
+            ),
+            (
+                Affine2D(0, -2.875, 2.875, 0, 2.39, 250.81),
+                Affine2D(0.0, 0.348, -0.348, 0.0, 87.238, -0.831),
+            ),
+        ],
+    )
+    def test_inverse(self, transform, inverse):
+        assert transform.inverse().almost_equals(inverse, 1e-3)
+        assert (transform @ transform.inverse()).almost_equals(Affine2D.identity())
+        assert (transform.inverse() @ transform).almost_equals(Affine2D.identity())
+
+    @pytest.mark.parametrize(
+        "transform",
+        [
+            Affine2D.identity().translate(2, 3).scale(4, 5),
+            Affine2D(0, -2.875, 2.875, 0, 2.39, 250.81),
+        ],
+    )
+    def test_inverse_roundtrip(self, transform):
         p0 = Point(12, 34)
-        p1 = t.map_point(p0)
-        it = t.inverse()
+        p1 = transform.map_point(p0)
+        it = transform.inverse()
         p2 = it.map_point(p1)
         assert p2 == p0
 
+    def test_inverse_degenerate(self):
         assert Affine2D.degenerate().inverse() == Affine2D.degenerate()
         t = Affine2D(1, 1, 1, 1, 0, 0).inverse()
         assert t.is_degenerate()
@@ -265,8 +290,6 @@ class TestAffine2D:
         p0 = Point(1, 1)
         expected = affine2.map_point(affine1.map_point(p0)).round(2)
         assert (affine2 @ affine1).map_point(p0).round(2) == expected
-        # deprecated
-        assert Affine2D.product(affine1, affine2).map_point(p0).round(2) == expected
 
     def test_product_ordering(self):
         affine1 = Affine2D.identity().rotate(pi / 2)
@@ -355,6 +378,21 @@ class TestAffine2D:
                 Affine2D(1, 0, 0, 1, 50, -100),
                 Affine2D(0, -1.0, 1.0, 0, 0, 0),
             ),
+            # https://github.com/googlefonts/picosvg/issues/239
+            (
+                Affine2D(
+                    a=-4.382842e-08,
+                    b=1.0027,
+                    c=-4.2262,
+                    d=-1.847346e-07,
+                    e=1133.4812,
+                    f=889.4797,
+                ),
+                Affine2D(a=1, b=0, c=0, d=1, e=887.0845192597527, f=-268.2034070511364),
+                Affine2D(
+                    a=-4.382842e-08, b=1.0027, c=-4.2262, d=-1.847346e-07, e=0, f=0
+                ),
+            ),
         ],
     )
     def test_decompose_translation(
